_G.spy = spy
local mock_env = require("spec.env_mock")
local rng = require("spec.fuzzgen")

describe("smr biography",function()
	setup(mock_env.setup)
	teardown(mock_env.teardown)
	it("should allow users to set their biography",function()
		local claim_post = require("endpoints.claim_post")
		local login_post = require("endpoints.login_post")
		local index_get = require("endpoints.index_get")
		local bio_get = require("endpoints.bio_get")
		local bio_post = require("endpoints.bio_post")
		local db = require("db")
		local config = require("config")
		config.domain = "test.host"
		configure()
		local username = rng.subdomain()
		local claim_req = {
			method = "POST",
			host = "test.host",
			path = "/_claim",
			args = {
				user = username
			}
		}
		claim_post(claim_req)
		local login_req = {
			method = "POST",
			host = "test.host",
			path = "/_login",
			args = {
				user = username
			},
			file = {
				pass = claim_req.response
			}
		}
		login_post(login_req)
		local cookie = login_req.response_headers["set-cookie"]
		local sessionid = cookie:match("session=([^;]+)")
		local home_req_get = {
			method = "GET",
			host = username .. ".test.host",
			path = "/",
			cookies = {
				session = sessionid
			}
		}
		index_get(home_req_get)
		local edit_bio_button = '<a href="/_bio"'
		assert(
			home_req_get.response:find(edit_bio_button),
			"After logging in the user should have a button to" ..
			" edit their biography. Looking for " .. edit_bio_button
			.. " but didn't find it in " .. home_req_get.response
		)
		local edit_bio_req_get = {
			method = "GET",
			host = username .. ".test.host",
			path = "/_bio",
			cookies = {
				session = sessionid
			},
			args = {}
		}
		bio_get(edit_bio_req_get)
		assert(edit_bio_req_get.responsecode == 200)
		--[=[
		local paste_req_post = {
			method = "POST",
			host = username .. ".test.host",
			path = "/_paste",
			cookies = {
				session = sessionid
			},
			args = {
				title = "post title",
				text = "post text",
				markup = "plain",
				tags = "",
			}
		}
		paste_post(paste_req_post)
		for row in db.conn:rows("SELECT COUNT(*) FROM posts") do
			assert(row[1] == 1, "Expected exactly 1 post in sample db")
		end
		local code = paste_req_post.responsecode
		assert(code >= 300 and code <= 400, "Should receive a redirect after posting, got:" .. tostring(code))
		assert(paste_req_post.response_headers, "Should have received some response headers")
		assert(paste_req_post.response_headers.Location, "Should have received a location in response headers")
		local redirect = paste_req_post.response_headers.Location:match("(/[^/]*)$")
		local read_req_get = {
			method = "GET",
			host = username .. ".test.host",
			path = redirect,
			cookies = {
				session = sessionid
			},
			args = {}
		}
		read_get(read_req_get)
		local response = read_req_get.response
		assert(
			response:find([[post title]]),
			"Failed to find post title in response."
		)
		assert(
			response:find('By <a href="https://' .. username .. '.test.host">' .. username .. '</a>'),
			"Failed to find the author name after a paste."
		)
		assert(
			response:find([[post text]]),
			"Failed to find post text in response."
		)
		]=]
	end)
end)
